@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 /*
  * Maps directory and song objects to file system paths.
@@ -24,7 +8,7 @@
 #include "config.h"
 #include "Mapper.hxx"
 #include "fs/AllocatedPath.hxx"
-#include "fs/CheckFile.hxx"
+#include "fs/glue/CheckFile.hxx"
 #include "util/StringCompare.hxx"
 
 #ifdef ENABLE_DATABASE
@@ -85,15 +69,15 @@ map_fs_to_utf8(Path path_fs) noexcept
 {
 	if (path_fs.IsAbsolute()) {
 		if (global_instance->storage == nullptr)
-			return std::string();
+			return {};
 
 		const auto music_dir_fs = global_instance->storage->MapFS("");
 		if (music_dir_fs.IsNull())
-			return std::string();
+			return {};
 
 		auto relative = music_dir_fs.Relative(path_fs);
 		if (relative == nullptr || StringIsEmpty(relative))
-			return std::string();
+			return {};
 
 		path_fs = Path::FromFS(relative);
 	}
