@@ -1,27 +1,13 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "SongSave.hxx"
 #include "pcm/AudioParser.hxx"
 #include "db/plugins/simple/Song.hxx"
 #include "song/DetachedSong.hxx"
 #include "TagSave.hxx"
+#include "lib/fmt/AudioFormatFormatter.hxx"
+#include "lib/fmt/RuntimeError.hxx"
 #include "io/LineReader.hxx"
 #include "io/BufferedOutputStream.hxx"
 #include "tag/ParseName.hxx"
@@ -31,60 +17,68 @@
 #include "util/StringAPI.hxx"
 #include "util/StringBuffer.hxx"
 #include "util/StringStrip.hxx"
-#include "util/RuntimeError.hxx"
-#include "util/NumberParser.hxx"
+#include "util/CNumberParser.hxx"
 
 #include <stdlib.h>
 
 #define SONG_MTIME "mtime"
+#define SONG_ADDED "added"
 #define SONG_END "song_end"
 
 static void
 range_save(BufferedOutputStream &os, unsigned start_ms, unsigned end_ms)
 {
 	if (end_ms > 0)
-		os.Format("Range: %u-%u\n", start_ms, end_ms);
+		os.Fmt("Range: {}-{}\n", start_ms, end_ms);
 	else if (start_ms > 0)
-		os.Format("Range: %u-\n", start_ms);
+		os.Fmt("Range: {}-\n", start_ms);
 }
 
 void
 song_save(BufferedOutputStream &os, const Song &song)
 {
-	os.Format(SONG_BEGIN "%s\n", song.filename.c_str());
+	os.Fmt(SONG_BEGIN "{}\n", song.filename);
 
 	if (!song.target.empty())
-		os.Format("Target: %s\n", song.target.c_str());
+		os.Fmt("Target: {}\n", song.target);
 
 	range_save(os, song.start_time.ToMS(), song.end_time.ToMS());
 
 	tag_save(os, song.tag);
 
 	if (song.audio_format.IsDefined())
-		os.Format("Format: %s\n", ToString(song.audio_format).c_str());
+		os.Fmt("Format: {}\n", song.audio_format);
 
 	if (song.in_playlist)
 		os.Write("InPlaylist: yes\n");
 
 	if (!IsNegative(song.mtime))
-		os.Format(SONG_MTIME ": %li\n",
-			  (long)std::chrono::system_clock::to_time_t(song.mtime));
-	os.Format(SONG_END "\n");
+		os.Fmt(SONG_MTIME ": {}\n",
+		       std::chrono::system_clock::to_time_t(song.mtime));
+
+	if (!IsNegative(song.added))
+		os.Fmt(SONG_ADDED ": {}\n",
+		       std::chrono::system_clock::to_time_t(song.added));
+	os.Write(SONG_END "\n");
 }
 
 void
 song_save(BufferedOutputStream &os, const DetachedSong &song)
 {
-	os.Format(SONG_BEGIN "%s\n", song.GetURI());
+	os.Fmt(SONG_BEGIN "{}\n", song.GetURI());
 
 	range_save(os, song.GetStartTime().ToMS(), song.GetEndTime().ToMS());
 
 	tag_save(os, song.GetTag());
 
 	if (!IsNegative(song.GetLastModified()))
-		os.Format(SONG_MTIME ": %li\n",
-			  (long)std::chrono::system_clock::to_time_t(song.GetLastModified()));
-	os.Format(SONG_END "\n");
+		os.Fmt(SONG_MTIME ": {}\n",
+		       std::chrono::system_clock::to_time_t(song.GetLastModified()));
+
+	if (!IsNegative(song.GetAdded()))
+		os.Fmt(SONG_ADDED ": {}\n",
+		       std::chrono::system_clock::to_time_t(song.GetAdded()));
+	os.Write(SONG_END "\n");
 }
 
 DetachedSong
@@ -99,16 +93,15 @@ song_load(LineReader &file, const char *uri,
 	while ((line = file.ReadLine()) != nullptr &&
 	       !StringIsEqual(line, SONG_END)) {
 		char *colon = std::strchr(line, ':');
-		if (colon == nullptr || colon == line) {
-			throw FormatRuntimeError("unknown line in db: %s", line);
-		}
+		if (colon == nullptr || colon == line)
+			throw FmtRuntimeError("unknown line in db: {}", line);
 
 		*colon++ = 0;
 		const char *value = StripLeft(colon);
 
 		TagType type;
 		if ((type = tag_name_parse(line)) != TAG_NUM_OF_ITEM_TYPES) {
-			tag.AddItem(type, value);
+			tag.AddItemUnchecked(type, value);
 		} else if (StringIsEqual(line, "Time")) {
 			tag.SetDuration(SignedSongTime::FromS(ParseDouble(value)));
 		} else if (StringIsEqual(line, "Target")) {
@@ -125,6 +118,8 @@ song_load(LineReader &file, const char *uri,
 			tag.SetHasPlaylist(StringIsEqual(value, "yes"));
 		} else if (StringIsEqual(line, SONG_MTIME)) {
 			song.SetLastModified(std::chrono::system_clock::from_time_t(atoi(value)));
+		} else if (StringIsEqual(line, SONG_ADDED)) {
+			song.SetAdded(std::chrono::system_clock::from_time_t(atoi(value)));
 		} else if (StringIsEqual(line, "Range")) {
 			char *endptr;
 
@@ -139,7 +134,7 @@ song_load(LineReader &file, const char *uri,
 			if (in_playlist_r != nullptr)
 				*in_playlist_r = StringIsEqual(value, "yes");
 		} else {
-			throw FormatRuntimeError("unknown line in db: %s", line);
+			throw FmtRuntimeError("unknown line in db: {}", line);
 		}
 	}
 
