@@ -1,33 +1,16 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "config.h"
 #include "StateFile.hxx"
 #include "output/State.hxx"
 #include "queue/PlaylistState.hxx"
-#include "fs/io/TextFile.hxx"
-#include "fs/io/FileOutputStream.hxx"
-#include "fs/io/BufferedOutputStream.hxx"
+#include "io/FileLineReader.hxx"
+#include "io/FileOutputStream.hxx"
+#include "io/BufferedOutputStream.hxx"
 #include "storage/StorageState.hxx"
 #include "Partition.hxx"
 #include "Instance.hxx"
-#include "mixer/Volume.hxx"
 #include "SongLoader.hxx"
 #include "util/Domain.hxx"
 #include "Log.hxx"
@@ -47,7 +30,7 @@ StateFile::StateFile(StateFileConfig &&_config,
 void
 StateFile::RememberVersions() noexcept
 {
-	prev_volume_version = sw_volume_state_get_hash();
+	prev_volume_version = partition.mixer_memento.GetSoftwareVolumeStateHash();
 	prev_output_version = audio_output_state_get_version();
 	prev_playlist_version = playlist_state_get_hash(partition.playlist,
 							partition.pc);
@@ -59,7 +42,7 @@ StateFile::RememberVersions() noexcept
 bool
 StateFile::IsModified() const noexcept
 {
-	return prev_volume_version != sw_volume_state_get_hash() ||
+	return prev_volume_version != partition.mixer_memento.GetSoftwareVolumeStateHash() ||
 		prev_output_version != audio_output_state_get_version() ||
 		prev_playlist_version != playlist_state_get_hash(partition.playlist,
 								 partition.pc)
@@ -72,7 +55,7 @@ StateFile::IsModified() const noexcept
 inline void
 StateFile::Write(BufferedOutputStream &os)
 {
-	save_sw_volume_state(os);
+	partition.mixer_memento.SaveSoftwareVolumeState(os);
 	audio_output_state_save(os, partition.outputs);
 
 #ifdef ENABLE_DATABASE
@@ -114,7 +97,7 @@ try {
 
 	FmtDebug(state_file_domain, "Loading state file {}", path_utf8);
 
-	TextFile file(config.path);
+	FileLineReader file{config.path};
 
 #ifdef ENABLE_DATABASE
 	const SongLoader song_loader(partition.instance.GetDatabase(),
@@ -125,7 +108,7 @@ try {
 
 	const char *line;
 	while ((line = file.ReadLine()) != nullptr) {
-		success = read_sw_volume_state(line, partition.outputs) ||
+		success = partition.mixer_memento.LoadSoftwareVolumeState(line, partition.outputs) ||
 			audio_output_state_read(line, partition.outputs) ||
 			playlist_state_restore(config, line, file, song_loader,
 					       partition.playlist,
