@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "OutputCommands.hxx"
 #include "Request.hxx"
@@ -24,16 +8,20 @@
 #include "client/Client.hxx"
 #include "client/Response.hxx"
 #include "Partition.hxx"
-#include "IdleFlags.hxx"
+#include "protocol/IdleFlags.hxx"
 #include "util/CharUtil.hxx"
+#include "util/StringVerify.hxx"
 
 CommandResult
 handle_enableoutput(Client &client, Request args, Response &r)
 {
-	assert(args.size == 1);
+	assert(args.size() == 1);
 	unsigned device = args.ParseUnsigned(0);
 
-	if (!audio_output_enable_index(client.GetPartition().outputs, device)) {
+	auto &partition = client.GetPartition();
+
+	if (!audio_output_enable_index(partition,
+				       device)) {
 		r.Error(ACK_ERROR_NO_EXIST, "No such audio output");
 		return CommandResult::ERROR;
 	}
@@ -44,10 +32,13 @@ handle_enableoutput(Client &client, Request args, Response &r)
 CommandResult
 handle_disableoutput(Client &client, Request args, Response &r)
 {
-	assert(args.size == 1);
+	assert(args.size() == 1);
 	unsigned device = args.ParseUnsigned(0);
 
-	if (!audio_output_disable_index(client.GetPartition().outputs, device)) {
+	auto &partition = client.GetPartition();
+
+	if (!audio_output_disable_index(partition,
+					device)) {
 		r.Error(ACK_ERROR_NO_EXIST, "No such audio output");
 		return CommandResult::ERROR;
 	}
@@ -58,10 +49,13 @@ handle_disableoutput(Client &client, Request args, Response &r)
 CommandResult
 handle_toggleoutput(Client &client, Request args, Response &r)
 {
-	assert(args.size == 1);
+	assert(args.size() == 1);
 	unsigned device = args.ParseUnsigned(0);
 
-	if (!audio_output_toggle_index(client.GetPartition().outputs, device)) {
+	auto &partition = client.GetPartition();
+
+	if (!audio_output_toggle_index(partition,
+				       device)) {
 		r.Error(ACK_ERROR_NO_EXIST, "No such audio output");
 		return CommandResult::ERROR;
 	}
@@ -69,28 +63,22 @@ handle_toggleoutput(Client &client, Request args, Response &r)
 	return CommandResult::OK;
 }
 
-static bool
+static constexpr bool
 IsValidAttributeNameChar(char ch) noexcept
 {
 	return IsAlphaNumericASCII(ch) || ch == '_';
 }
 
-gcc_pure
-static bool
+static constexpr bool
 IsValidAttributeName(const char *s) noexcept
 {
-	do {
-		if (!IsValidAttributeNameChar(*s))
-			return false;
-	} while (*++s);
-
-	return true;
+	return CheckCharsNonEmpty(s, IsValidAttributeNameChar);
 }
 
 CommandResult
 handle_outputset(Client &client, Request request, Response &response)
 {
-	assert(request.size == 3);
+	assert(request.size() == 3);
 	const unsigned i = request.ParseUnsigned(0);
 
 	auto &partition = client.GetPartition();
