@@ -1,26 +1,10 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "Data.hxx"
 #include "Parser.hxx"
 #include "fs/AllocatedPath.hxx"
-#include "util/RuntimeError.hxx"
+#include "lib/fmt/RuntimeError.hxx"
 #include "util/StringAPI.hxx"
 
 #include <stdlib.h>
@@ -36,7 +20,7 @@ ConfigData::Clear()
 }
 
 template<typename T>
-gcc_pure
+[[gnu::pure]]
 static auto
 FindLast(const std::forward_list<T> &list)
 {
@@ -102,32 +86,17 @@ ConfigData::GetPositive(ConfigOption option, unsigned default_value) const
 }
 
 std::chrono::steady_clock::duration
-ConfigData::GetUnsigned(ConfigOption option,
+ConfigData::GetDuration(ConfigOption option,
+			std::chrono::steady_clock::duration min_value,
 			std::chrono::steady_clock::duration default_value) const
 {
-	return With(option, [default_value](const char *s){
+	return With(option, [min_value, default_value](const char *s){
 		if (s == nullptr)
 			return default_value;
 
 		auto value = ParseDuration(s);
-		if (value < std::chrono::steady_clock::duration{})
-			throw std::runtime_error("Value must not be negative");
-
-		return value;
-	});
-}
-
-std::chrono::steady_clock::duration
-ConfigData::GetPositive(ConfigOption option,
-			std::chrono::steady_clock::duration default_value) const
-{
-	return With(option, [default_value](const char *s){
-		if (s == nullptr)
-			return default_value;
-
-		auto value = ParseDuration(s);
-		if (value <= std::chrono::steady_clock::duration{})
-			throw std::runtime_error("Value must be positive");
+		if (value < min_value)
+			throw std::runtime_error{"Value is too small"};
 
 		return value;
 	});
@@ -157,8 +126,8 @@ ConfigData::FindBlock(ConfigBlockOption option,
 	for (const auto &block : GetBlockList(option)) {
 		const char *value2 = block.GetBlockValue(key);
 		if (value2 == nullptr)
-			throw FormatRuntimeError("block without '%s' in line %d",
-						 key, block.line);
+			throw FmtRuntimeError("block without {:?} in line {}",
+					      key, block.line);
 
 		if (StringIsEqual(value2, value))
 			return &block;
