@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "Templates.hxx"
 #include "Option.hxx"
@@ -73,9 +57,7 @@ const ConfigTemplate config_param_templates[] = {
 	{ "gapless_mp3_playback", false, true },
 	{ "auto_update" },
 	{ "auto_update_depth" },
-	{ "despotify_user", false, true },
-	{ "despotify_password", false, true },
-	{ "despotify_high_bitrate", false, true },
+	{ "mixramp_analyzer" },
 };
 
 static constexpr unsigned n_config_param_templates =
@@ -89,11 +71,13 @@ const ConfigTemplate config_block_templates[] = {
 	{ "decoder", true },
 	{ "input", true },
 	{ "input_cache" },
+	{ "archive_plugin", true },
 	{ "playlist_plugin", true },
 	{ "resampler" },
 	{ "filter", true },
 	{ "database" },
 	{ "neighbors", true },
+	{ "partition", true },
 };
 
 static constexpr unsigned n_config_block_templates =
@@ -102,7 +86,7 @@ static constexpr unsigned n_config_block_templates =
 static_assert(n_config_block_templates == unsigned(ConfigBlockOption::MAX),
 	      "Wrong number of config_block_templates");
 
-gcc_pure
+[[gnu::pure]]
 static inline unsigned
 ParseConfigTemplateName(const ConfigTemplate templates[], unsigned count,
 			const char *name) noexcept
