@@ -1,40 +1,37 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 /* necessary because libavutil/common.h uses UINT64_C */
 #define __STDC_CONSTANT_MACROS
 
 #include "FfmpegMetaData.hxx"
+#include "tag/Names.hxx"
 #include "tag/Table.hxx"
 #include "tag/Handler.hxx"
 #include "tag/Id3MusicBrainz.hxx"
-#include "util/StringView.hxx"
 
 extern "C" {
 #include <libavutil/dict.h>
 }
 
+/**
+ * FFmpeg specific tag name mappings, as supported by
+ * libavformat/id3v2.c, libavformat/mov.c and others.
+ */
 static constexpr struct tag_table ffmpeg_tags[] = {
-	{ "year", TAG_DATE },
-	{ "author-sort", TAG_ARTIST_SORT },
+	/* from libavformat/id3v2.c, libavformat/mov.c */
 	{ "album_artist", TAG_ALBUM_ARTIST },
-	{ "album_artist-sort", TAG_ALBUM_ARTIST_SORT },
+
+	/* from libavformat/id3v2.c */
+	{ "album-sort", TAG_ALBUM_SORT },
+	{ "artist-sort", TAG_ARTIST_SORT },
+	{ "title-sort", TAG_TITLE_SORT},
+
+	/* from libavformat/mov.c */
+	{ "sort_album_artist", TAG_ALBUM_ARTIST_SORT },
+	{ "sort_album", TAG_ALBUM_SORT },
+	{ "sort_artist", TAG_ARTIST_SORT },
+	{ "sort_name", TAG_TITLE_SORT },
 
 	/* sentinel */
 	{ nullptr, TAG_NUM_OF_ITEM_TYPES }
