@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "SndfileDecoderPlugin.hxx"
 #include "../DecoderAPI.hxx"
@@ -24,7 +8,6 @@
 #include "tag/Handler.hxx"
 #include "util/Domain.hxx"
 #include "util/ScopeExit.hxx"
-#include "util/StringView.hxx"
 #include "Log.hxx"
 
 #include <exception>
@@ -47,7 +30,8 @@ struct SndfileInputStream {
 	size_t Read(void *buffer, size_t size) {
 		/* libsndfile chokes on partial reads; therefore
 		   always force full reads */
-		return decoder_read_much(client, is, buffer, size);
+		return decoder_read_much(client, is,
+					 {reinterpret_cast<std::byte *>(buffer), size});
 	}
 };
 
@@ -145,7 +129,7 @@ sndfile_duration(const SF_INFO &info)
 	return SongTime::FromScale<uint64_t>(info.frames, info.samplerate);
 }
 
-gcc_pure
+[[gnu::pure]]
 static SampleFormat
 sndfile_sample_format(const SF_INFO &info) noexcept
 {
@@ -228,9 +212,9 @@ sndfile_stream_decode(DecoderClient &client, InputStream &is)
 		if (num_frames <= 0)
 			break;
 
-		cmd = client.SubmitData(is,
-					buffer, num_frames * frame_size,
-					0);
+		cmd = client.SubmitAudio(is,
+					 std::span{buffer, static_cast<size_t>(num_frames) * frame_size},
+					 0);
 		if (cmd == DecoderCommand::SEEK) {
 			sf_count_t c = client.GetSeekFrame();
 			c = sf_seek(sf, c, SEEK_SET);
