@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "config.h"
 #include "EncoderList.hxx"
@@ -33,7 +17,7 @@
 
 #include <string.h>
 
-const EncoderPlugin *const encoder_plugins[] = {
+constinit const EncoderPlugin *const encoder_plugins[] = {
 	&null_encoder_plugin,
 #ifdef ENABLE_VORBISENC
 	&vorbis_encoder_plugin,
@@ -62,9 +46,10 @@ const EncoderPlugin *const encoder_plugins[] = {
 const EncoderPlugin *
 encoder_plugin_get(const char *name)
 {
-	encoder_plugins_for_each(plugin)
-		if (strcmp(plugin->name, name) == 0)
-			return plugin;
+	for (const auto &plugin : GetAllEncoderPlugins()) {
+		if (strcmp(plugin.name, name) == 0)
+			return &plugin;
+	}
 
 	return nullptr;
 }
