@@ -1,37 +1,21 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "FlacEncoderPlugin.hxx"
 #include "../EncoderAPI.hxx"
+#include "tag/Names.hxx"
 #include "pcm/AudioFormat.hxx"
 #include "pcm/Buffer.hxx"
+#include "lib/fmt/RuntimeError.hxx"
 #include "util/DynamicFifoBuffer.hxx"
-#include "util/RuntimeError.hxx"
 #include "util/Serial.hxx"
+#include "util/SpanCast.hxx"
 #include "util/StringUtil.hxx"
 
 #include <FLAC/stream_encoder.h>
 #include <FLAC/metadata.h>
 
-#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
-#error libFLAC is too old
-#endif
+#include <algorithm>
 
 class FlacEncoder final : public Encoder {
 	const AudioFormat audio_format;
@@ -44,9 +28,9 @@ class FlacEncoder final : public Encoder {
 
 	/**
 	 * This buffer will hold encoded data from libFLAC until it is
-	 * picked up with flac_encoder_read().
+	 * picked up with Read().
 	 */
-	DynamicFifoBuffer<uint8_t> output_buffer;
+	DynamicFifoBuffer<std::byte> output_buffer{8192};
 
 public:
 	FlacEncoder(AudioFormat _audio_format, FLAC__StreamEncoder *_fse, unsigned _compression, bool _oggflac, bool _oggchaining);
@@ -72,10 +56,12 @@ public:
 
 	void SendTag(const Tag &tag) override;
 
-	void Write(const void *data, size_t length) override;
+	void Write(std::span<const std::byte> src) override;
 
-	size_t Read(void *dest, size_t length) noexcept override {
-		return output_buffer.Read((uint8_t *)dest, length);
+	std::span<const std::byte> Read(std::span<std::byte>) noexcept override {
+		auto r = output_buffer.Read();
+		output_buffer.Consume(r.size());
+		return r;
 	}
 
 private:
@@ -86,7 +72,7 @@ private:
 							    [[maybe_unused]] unsigned current_frame,
 							    void *client_data) noexcept {
 		auto &encoder = *(FlacEncoder *)client_data;
-		encoder.output_buffer.Append((const uint8_t *)data, bytes);
+		encoder.output_buffer.Append({(const std::byte *)data, bytes});
 		return FLAC__STREAM_ENCODER_WRITE_STATUS_OK;
 	}
 };
@@ -142,33 +128,32 @@ flac_encoder_setup(FLAC__StreamEncoder *fse, unsigned compression, bool oggflac,
 	}
 
 	if (!FLAC__stream_encoder_set_compression_level(fse, compression))
-		throw FormatRuntimeError("error setting flac compression to %d",
-					 compression);
+		throw FmtRuntimeError("error setting flac compression to {}",
+				      compression);
 
 	if (!FLAC__stream_encoder_set_channels(fse, audio_format.channels))
-		throw FormatRuntimeError("error setting flac channels num to %d",
-					 audio_format.channels);
+		throw FmtRuntimeError("error setting flac channels num to {}",
+				      audio_format.channels);
 
 	if (!FLAC__stream_encoder_set_bits_per_sample(fse, bits_per_sample))
-		throw FormatRuntimeError("error setting flac bit format to %d",
-					 bits_per_sample);
+		throw FmtRuntimeError("error setting flac bit format to {}",
+				      bits_per_sample);
 
 	if (!FLAC__stream_encoder_set_sample_rate(fse,
 						  audio_format.sample_rate))
-		throw FormatRuntimeError("error setting flac sample rate to %d",
-					 audio_format.sample_rate);
+		throw FmtRuntimeError("error setting flac sample rate to {}",
+				      audio_format.sample_rate);
 
 	if (oggflac && !FLAC__stream_encoder_set_ogg_serial_number(fse,
 						  GenerateSerial()))
-		throw FormatRuntimeError("error setting ogg serial number");
+		throw std::runtime_error{"error setting ogg serial number"};
 }
 
 FlacEncoder::FlacEncoder(AudioFormat _audio_format, FLAC__StreamEncoder *_fse, unsigned _compression, bool _oggflac, bool _oggchaining)
 	:Encoder(_oggchaining),
 	 audio_format(_audio_format), fse(_fse),
 	 compression(_compression),
-	 oggflac(_oggflac),
-	 output_buffer(8192)
+	 oggflac(_oggflac)
 {
 	/* this immediately outputs data through callback */
 
@@ -184,8 +169,8 @@ FlacEncoder::FlacEncoder(AudioFormat _audio_format, FLAC__StreamEncoder *_fse, u
 						 this);
 
 	if (init_status != FLAC__STREAM_ENCODER_INIT_STATUS_OK)
-		throw FormatRuntimeError("failed to initialize encoder: %s\n",
-					 FLAC__StreamEncoderInitStatusString[init_status]);
+		throw FmtRuntimeError("failed to initialize encoder: {}",
+				      FLAC__StreamEncoderInitStatusString[init_status]);
 }
 
 Encoder *
@@ -246,71 +231,52 @@ FlacEncoder::SendTag(const Tag &tag)
 	FLAC__metadata_object_delete(metadata);
 
 	if (init_status != FLAC__STREAM_ENCODER_INIT_STATUS_OK)
-		throw FormatRuntimeError("failed to initialize encoder: %s\n",
-					 FLAC__StreamEncoderInitStatusString[init_status]);
+		throw FmtRuntimeError("failed to initialize encoder: {}",
+				      FLAC__StreamEncoderInitStatusString[init_status]);
 }
 
-
-static inline void
-pcm8_to_flac(int32_t *out, const int8_t *in, unsigned num_samples) noexcept
+template<typename T>
+static std::span<const FLAC__int32>
+ToFlac32(PcmBuffer &buffer, std::span<const T> src) noexcept
 {
-	while (num_samples > 0) {
-		*out++ = *in++;
-		--num_samples;
-	}
+	FLAC__int32 *dest = buffer.GetT<FLAC__int32>(src.size());
+	std::copy(src.begin(), src.end(), dest);
+	return {dest, src.size()};
 }
 
-static inline void
-pcm16_to_flac(int32_t *out, const int16_t *in, unsigned num_samples) noexcept
+static std::span<const FLAC__int32>
+ToFlac32(PcmBuffer &buffer, std::span<const std::byte> src,
+	 SampleFormat format)
 {
-	while (num_samples > 0) {
-		*out++ = *in++;
-		--num_samples;
-	}
-}
-
-void
-FlacEncoder::Write(const void *data, size_t length)
-{
-	void *exbuffer;
-	const void *buffer = nullptr;
-
-	/* format conversion */
-
-	const unsigned num_frames = length / audio_format.GetFrameSize();
-	const unsigned num_samples = num_frames * audio_format.channels;
-
-	switch (audio_format.format) {
+	switch (format) {
 	case SampleFormat::S8:
-		exbuffer = expand_buffer.Get(length * 4);
-		pcm8_to_flac((int32_t *)exbuffer, (const int8_t *)data,
-			     num_samples);
-		buffer = exbuffer;
-		break;
+		return ToFlac32(buffer, FromBytesStrict<const int8_t>(src));
 
 	case SampleFormat::S16:
-		exbuffer = expand_buffer.Get(length * 2);
-		pcm16_to_flac((int32_t *)exbuffer, (const int16_t *)data,
-			      num_samples);
-		buffer = exbuffer;
-		break;
+		return ToFlac32(buffer, FromBytesStrict<const int16_t>(src));
 
 	case SampleFormat::S24_P32:
 	case SampleFormat::S32:
 		/* nothing need to be done; format is the same for
 		   both mpd and libFLAC */
-		buffer = data;
-		break;
+		return FromBytesStrict<const int32_t>(src);
 
 	default:
 		gcc_unreachable();
 	}
+}
+
+void
+FlacEncoder::Write(std::span<const std::byte> src)
+{
+	const auto imported = ToFlac32(expand_buffer, src,
+				       audio_format.format);
+	const std::size_t n_frames = imported.size() / audio_format.channels;
 
 	/* feed samples to encoder */
 
-	if (!FLAC__stream_encoder_process_interleaved(fse,
-						      (const FLAC__int32 *)buffer,
-						      num_frames))
+	if (!FLAC__stream_encoder_process_interleaved(fse, imported.data(),
+						      n_frames))
 		throw std::runtime_error("flac encoder process failed");
 }
 
