@@ -1,24 +1,9 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "WaveEncoderPlugin.hxx"
 #include "../EncoderAPI.hxx"
+#include "tag/RiffFormat.hxx"
 #include "util/ByteOrder.hxx"
 #include "util/DynamicFifoBuffer.hxx"
 
@@ -26,21 +11,19 @@
 
 #include <string.h>
 
-static constexpr uint16_t WAVE_FORMAT_PCM = 1;
-
 class WaveEncoder final : public Encoder {
 	unsigned bits;
 
-	DynamicFifoBuffer<uint8_t> buffer;
+	DynamicFifoBuffer<std::byte> buffer{8192};
 
 public:
 	explicit WaveEncoder(AudioFormat &audio_format) noexcept;
 
 	/* virtual methods from class Encoder */
-	void Write(const void *data, size_t length) override;
+	void Write(std::span<const std::byte> src) override;
 
-	size_t Read(void *dest, size_t length) noexcept override {
-		return buffer.Read((uint8_t *)dest, length);
+	std::span<const std::byte> Read(std::span<std::byte> b) noexcept override {
+		return b.first(buffer.Read(b.data(), b.size()));
 	}
 };
 
@@ -56,45 +39,44 @@ class PreparedWaveEncoder final : public PreparedEncoder {
 };
 
 struct WaveHeader {
-	uint32_t id_riff;
-	uint32_t riff_size;
-	uint32_t id_wave;
-	uint32_t id_fmt;
-	uint32_t fmt_size;
-	uint16_t format;
-	uint16_t channels;
-	uint32_t freq;
-	uint32_t byterate;
-	uint16_t blocksize;
-	uint16_t bits;
-	uint32_t id_data;
-	uint32_t data_size;
+	RiffFileHeader file_header;
+	RiffChunkHeader fmt_header;
+	RiffFmtChunk fmt;
+	RiffChunkHeader data_header;
 };
 
-static void
-fill_wave_header(WaveHeader *header, int channels, int bits,
-		int freq, int block_size) noexcept
+static_assert(sizeof(WaveHeader) == 44);
+
+static WaveHeader
+MakeWaveHeader(int channels, int bits,
+	       int freq, int block_size) noexcept
 {
+	WaveHeader header{};
+
 	int data_size = 0x0FFFFFFF;
 
 	/* constants */
-	header->id_riff = ToLE32(0x46464952);
-	header->id_wave = ToLE32(0x45564157);
-	header->id_fmt = ToLE32(0x20746d66);
-	header->id_data = ToLE32(0x61746164);
+	memcpy(header.file_header.id, "RIFF", 4);
+	memcpy(header.file_header.format, "WAVE", 4);
+	memcpy(header.fmt_header.id, "fmt ", 4);
+	memcpy(header.data_header.id, "data", 4);
 
 	/* wave format */
-	header->format = ToLE16(WAVE_FORMAT_PCM);
-	header->channels = ToLE16(channels);
-	header->bits = ToLE16(bits);
-	header->freq = ToLE32(freq);
-	header->blocksize = ToLE16(block_size);
-	header->byterate = ToLE32(freq * block_size);
+	header.fmt.tag = ToLE16(RiffFmtChunk::TAG_PCM);
+	header.fmt.channels = ToLE16(channels);
+	header.fmt.bits_per_sample = ToLE16(bits);
+	header.fmt.sample_rate = ToLE32(freq);
+	header.fmt.block_align = ToLE16(block_size);
+	header.fmt.byte_rate = ToLE32(freq * block_size);
 
 	/* chunk sizes (fake data length) */
-	header->fmt_size = ToLE32(16);
-	header->data_size = ToLE32(data_size);
-	header->riff_size = ToLE32(4 + (8 + 16) + (8 + data_size));
+	header.fmt_header.size = ToLE32(sizeof(header.fmt));
+	header.data_header.size = ToLE32(data_size);
+	header.file_header.size = ToLE32(4 +
+					 sizeof(header.fmt_header) + sizeof(header.fmt) +
+					 sizeof(header.data_header) + data_size);
+
+	return header;
 }
 
 static PreparedEncoder *
@@ -104,8 +86,7 @@ wave_encoder_init([[maybe_unused]] const ConfigBlock &block)
 }
 
 WaveEncoder::WaveEncoder(AudioFormat &audio_format) noexcept
-	:Encoder(false),
-	 buffer(8192)
+	:Encoder(false)
 {
 	assert(audio_format.IsValid());
 
@@ -133,15 +114,14 @@ WaveEncoder::WaveEncoder(AudioFormat &audio_format) noexcept
 	}
 
 	auto range = buffer.Write();
-	assert(range.size >= sizeof(WaveHeader));
-	auto *header = (WaveHeader *)range.data;
+	assert(range.size() >= sizeof(WaveHeader));
+	auto *header = (WaveHeader *)(void *)range.data();
 
 	/* create PCM wave header in initial buffer */
-	fill_wave_header(header,
-			 audio_format.channels,
-			 bits,
-			 audio_format.sample_rate,
-			 (bits / 8) * audio_format.channels);
+	*header = MakeWaveHeader(audio_format.channels,
+				 bits,
+				 audio_format.sample_rate,
+				 (bits / 8) * audio_format.channels);
 
 	buffer.Append(sizeof(*header));
 }
@@ -187,36 +167,43 @@ pcm24_to_wave(uint8_t *dst8, const uint32_t *src32, size_t length) noexcept
 }
 
 void
-WaveEncoder::Write(const void *src, size_t length)
+WaveEncoder::Write(std::span<const std::byte> src)
 {
-	uint8_t *dst = buffer.Write(length);
+	std::size_t length = src.size();
+	std::byte *dst = buffer.Write(length);
 
 	if (IsLittleEndian()) {
 		switch (bits) {
 		case 8:
 		case 16:
 		case 32:// optimized cases
-			memcpy(dst, src, length);
+			memcpy(dst, src.data(), length);
 			break;
 		case 24:
-			length = pcm24_to_wave(dst, (const uint32_t *)src, length);
+			length = pcm24_to_wave((uint8_t *)dst,
+					       (const uint32_t *)(const void *)src.data(),
+					       length);
 			break;
 		}
 	} else {
 		switch (bits) {
 		case 8:
-			memcpy(dst, src, length);
+			memcpy(dst, src.data(), length);
 			break;
 		case 16:
 			length = pcm16_to_wave((uint16_t *)dst,
-					       (const uint16_t *)src, length);
+					       (const uint16_t *)(const void *)src.data(),
+					       length);
 			break;
 		case 24:
-			length = pcm24_to_wave(dst, (const uint32_t *)src, length);
+			length = pcm24_to_wave((uint8_t *)dst,
+					       (const uint32_t *)(const void *)src.data(),
+					       length);
 			break;
 		case 32:
 			length = pcm32_to_wave((uint32_t *)dst,
-					       (const uint32_t *)src, length);
+					       (const uint32_t *)(const void *)src.data(),
+					       length);
 			break;
 		}
 	}
