@@ -1,35 +1,18 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "FullyBufferedSocket.hxx"
 #include "net/SocketError.hxx"
-#include "util/Compiler.h"
 
 #include <cassert>
 
 #include <string.h>
 
-FullyBufferedSocket::ssize_t
-FullyBufferedSocket::DirectWrite(const void *data, size_t length) noexcept
+inline FullyBufferedSocket::ssize_t
+FullyBufferedSocket::DirectWrite(std::span<const std::byte> src) noexcept
 {
-	const auto nbytes = GetSocket().Write((const char *)data, length);
-	if (gcc_unlikely(nbytes < 0)) {
+	const auto nbytes = GetSocket().WriteNoWait(src);
+	if (nbytes < 0) [[unlikely]] {
 		const auto code = GetSocketError();
 		if (IsSocketErrorSendWouldBlock(code))
 			return 0;
@@ -58,8 +41,8 @@ FullyBufferedSocket::Flush() noexcept
 		return true;
 	}
 
-	auto nbytes = DirectWrite(data.data, data.size);
-	if (gcc_unlikely(nbytes <= 0))
+	auto nbytes = DirectWrite(data);
+	if (nbytes <= 0) [[unlikely]]
 		return nbytes == 0;
 
 	output.Consume(nbytes);
@@ -82,7 +65,7 @@ FullyBufferedSocket::Write(const void *data, size_t length) noexcept
 
 	const bool was_empty = output.empty();
 
-	if (!output.Append(data, length)) {
+	if (!output.Append({(const std::byte *)data, length})) {
 		OnSocketError(std::make_exception_ptr(std::runtime_error("Output buffer is full")));
 		return false;
 	}
