@@ -12,6 +12,8 @@
 #include "../InputPlugin.hxx"
 #include "util/StringAPI.hxx"
 
+using std::string_view_literals::operator""sv;
+
 class FfmpegInputStream final : public ThreadInputStream {
 	static constexpr std::size_t BUFFER_SIZE = 256 * 1024;
 
@@ -66,14 +68,14 @@ input_ffmpeg_protocols() noexcept
 		if (StringIsEqual(protocol, "hls")) {
 			/* just "hls://" doesn't work, but these do
 			   work: */
-			protocols.emplace("hls+http://");
-			protocols.emplace("hls+https://");
+			protocols.emplace("hls+http://"sv);
+			protocols.emplace("hls+https://"sv);
 			continue;
 		}
 
 		if (protocol_is_whitelisted(protocol)) {
 			std::string schema(protocol);
-			schema.append("://");
+			schema.append("://"sv);
 			protocols.emplace(schema);
 		}
 	}
