@@ -1,44 +1,30 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "AllowedFormat.hxx"
 #include "pcm/AudioParser.hxx"
 #include "util/IterableSplitString.hxx"
 #include "util/StringBuffer.hxx"
+#include "util/StringCompare.hxx"
 #include "config.h"
 
 #include <stdexcept>
 
+using std::string_view_literals::operator""sv;
+
 namespace Alsa {
 
-AllowedFormat::AllowedFormat(StringView s)
+AllowedFormat::AllowedFormat(std::string_view s)
 {
 #ifdef ENABLE_DSD
-	dop = s.RemoveSuffix("=dop");
+	dop = RemoveSuffix(s, "=dop"sv);
 #endif
 
 	char buffer[64];
-	if (s.size >= sizeof(buffer))
+	if (s.size() >= sizeof(buffer))
 		throw std::invalid_argument("Failed to parse audio format");
 
-	memcpy(buffer, s.data, s.size);
-	buffer[s.size] = 0;
+	*std::copy(s.begin(), s.end(), buffer) = 0;
 
 	format = ParseAudioFormat(buffer, true);
 
