@@ -1,29 +1,13 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "Collate.hxx"
 #include "util/AllocatedString.hxx"
 #include "config.h"
 
 #ifdef HAVE_ICU
+#include "Error.hxx"
 #include "Util.hxx"
-#include "util/RuntimeError.hxx"
 
 #include <unicode/ucol.h>
 #include <unicode/ustring.h>
@@ -62,8 +46,7 @@ IcuCollateInit()
 	UErrorCode code = U_ZERO_ERROR;
 	collator = ucol_open("", &code);
 	if (collator == nullptr)
-		throw FormatRuntimeError("ucol_open() failed: %s",
-					 u_errorName(code));
+		throw ICU::MakeError(code, "ucol_open() failed");
 }
 
 void
@@ -72,11 +55,12 @@ IcuCollateFinish() noexcept
 	assert(collator != nullptr);
 
 	ucol_close(collator);
+	collator = nullptr;
 }
 
 #endif
 
-gcc_pure
+[[gnu::pure]]
 int
 IcuCollate(std::string_view a, std::string_view b) noexcept
 {
@@ -88,7 +72,7 @@ IcuCollate(std::string_view a, std::string_view b) noexcept
 				     b.data(), b.size(), &code);
 
 #elif defined(_WIN32)
-	AllocatedString<wchar_t> wa = nullptr, wb = nullptr;
+	BasicAllocatedString<wchar_t> wa, wb;
 
 	try {
 		wa = MultiByteToWideChar(CP_UTF8, a);
