@@ -1,30 +1,14 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "PulseMixerPlugin.hxx"
+#include "lib/fmt/RuntimeError.hxx"
 #include "lib/pulse/LogError.hxx"
 #include "lib/pulse/LockGuard.hxx"
-#include "mixer/MixerInternal.hxx"
+#include "mixer/Mixer.hxx"
 #include "mixer/Listener.hxx"
 #include "output/plugins/PulseOutputPlugin.hxx"
-#include "util/NumberParser.hxx"
-#include "util/RuntimeError.hxx"
+#include "util/CNumberParser.hxx"
 #include "config/Block.hxx"
 
 #include <pulse/context.h>
@@ -55,6 +39,9 @@ public:
 
 	~PulseMixer() override;
 
+	PulseMixer(const PulseMixer &) = delete;
+	PulseMixer &operator=(const PulseMixer &) = delete;
+
 	void Offline();
 	void VolumeCallback(const pa_sink_input_info *i, int eol);
 	void Update(pa_context *context, pa_stream *stream);
@@ -174,9 +161,9 @@ parse_volume_scale_factor(const char *value) {
 	float factor = ParseFloat(value, &endptr);
 
 	if (endptr == value || *endptr != '\0' || factor < 0.5f || factor > 5.0f)
-		throw FormatRuntimeError("\"%s\" is not a number in the "
-					 "range 0.5 to 5.0",
-					 value);
+		throw FmtRuntimeError("{:?} is not a number in the "
+				      "range 0.5 to 5.0",
+				      value);
 
 	return factor;
 }
