@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 /*
  * Saving and loading the audio output states to/from the state file.
@@ -29,6 +13,8 @@
 #include "io/BufferedOutputStream.hxx"
 #include "util/StringCompare.hxx"
 
+#include <fmt/format.h>
+
 #include <stdlib.h>
 
 #define AUDIO_DEVICE_STATE "audio_device_state:"
@@ -41,10 +27,10 @@ audio_output_state_save(BufferedOutputStream &os,
 {
 	for (unsigned i = 0, n = outputs.Size(); i != n; ++i) {
 		const auto &ao = outputs.Get(i);
-		const std::scoped_lock<Mutex> lock(ao.mutex);
+		const std::scoped_lock lock{ao.mutex};
 
-		os.Format(AUDIO_DEVICE_STATE "%d:%s\n",
-			  ao.IsEnabled(), ao.GetName());
+		os.Fmt(AUDIO_DEVICE_STATE "{}:{}\n",
+		       (unsigned)ao.IsEnabled(), ao.GetName());
 	}
 }
 
@@ -71,7 +57,7 @@ audio_output_state_read(const char *line, MultipleOutputs &outputs)
 	auto *ao = outputs.FindByName(name);
 	if (ao == nullptr) {
 		FmtDebug(output_domain,
-			 "Ignoring device state for '{}'", name);
+			 "Ignoring device state for {:?}", name);
 		return true;
 	}
 
