@@ -1,26 +1,10 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "SolarisOutputPlugin.hxx"
 #include "../OutputAPI.hxx"
 #include "io/FileDescriptor.hxx"
-#include "system/Error.hxx"
+#include "lib/fmt/SystemError.hxx"
 
 #include <cerrno>
 
@@ -76,7 +60,7 @@ private:
 	void Open(AudioFormat &audio_format) override;
 	void Close() noexcept override;
 
-	size_t Play(const void *chunk, size_t size) override;
+	std::size_t Play(std::span<const std::byte> src) override;
 	void Cancel() noexcept override;
 };
 
@@ -100,8 +84,7 @@ SolarisOutput::Open(AudioFormat &audio_format)
 	/* open the device in non-blocking mode */
 
 	if (!fd.Open(device, O_WRONLY|O_NONBLOCK))
-		throw FormatErrno("Failed to open %s",
-				  device);
+		throw FmtErrno("Failed to open {}", device);
 
 	/* restore blocking mode */
 
@@ -139,10 +122,10 @@ SolarisOutput::Close() noexcept
 	fd.Close();
 }
 
-size_t
-SolarisOutput::Play(const void *chunk, size_t size)
+std::size_t
+SolarisOutput::Play(std::span<const std::byte> src)
 {
-	ssize_t nbytes = fd.Write(chunk, size);
+	ssize_t nbytes = fd.Write(src);
 	if (nbytes <= 0)
 		throw MakeErrno("Write failed");
 
