@@ -1,25 +1,9 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "Control.hxx"
 #include "Outputs.hxx"
-#include "Idle.hxx"
+#include "Listener.hxx"
 #include "song/DetachedSong.hxx"
 
 #include <algorithm>
@@ -28,15 +12,12 @@
 PlayerControl::PlayerControl(PlayerListener &_listener,
 			     PlayerOutputs &_outputs,
 			     InputCacheManager *_input_cache,
-			     unsigned _buffer_chunks,
-			     AudioFormat _configured_audio_format,
-			     const ReplayGainConfig &_replay_gain_config) noexcept
+			     const PlayerConfig &_config) noexcept
 	:listener(_listener), outputs(_outputs),
 	 input_cache(_input_cache),
-	 buffer_chunks(_buffer_chunks),
-	 configured_audio_format(_configured_audio_format),
-	 thread(BIND_THIS_METHOD(RunThread)),
-	 replay_gain_config(_replay_gain_config)
+	 config(_config),
+	 thread(BIND_THIS_METHOD(RunThread))
+
 {
 }
 
@@ -66,7 +47,7 @@ PlayerControl::Play(std::unique_ptr<DetachedSong> song)
 
 	assert(song != nullptr);
 
-	std::unique_lock<Mutex> lock(mutex);
+	std::unique_lock lock{mutex};
 	SeekLocked(lock, std::move(song), SongTime::zero());
 
 	if (state == PlayerState::PAUSE)
@@ -93,7 +74,7 @@ PlayerControl::LockStop() noexcept
 	LockSynchronousCommand(PlayerCommand::CLOSE_AUDIO);
 	assert(next_song == nullptr);
 
-	idle_add(IDLE_PLAYER);
+	listener.OnPlayerStateChanged();
 }
 
 void
@@ -114,7 +95,7 @@ PlayerControl::Kill() noexcept
 	LockSynchronousCommand(PlayerCommand::EXIT);
 	thread.Join();
 
-	idle_add(IDLE_PLAYER);
+	listener.OnPlayerStateChanged();
 }
 
 void
@@ -122,14 +103,14 @@ PlayerControl::PauseLocked(std::unique_lock<Mutex> &lock) noexcept
 {
 	if (state != PlayerState::STOP) {
 		SynchronousCommand(lock, PlayerCommand::PAUSE);
-		idle_add(IDLE_PLAYER);
+		listener.OnPlayerStateChanged();
 	}
 }
 
 void
 PlayerControl::LockPause() noexcept
 {
-	std::unique_lock<Mutex> lock(mutex);
+	std::unique_lock lock{mutex};
 	PauseLocked(lock);
 }
 
@@ -139,7 +120,7 @@ PlayerControl::LockSetPause(bool pause_flag) noexcept
 	if (!thread.IsDefined())
 		return;
 
-	std::unique_lock<Mutex> lock(mutex);
+	std::unique_lock lock{mutex};
 
 	switch (state) {
 	case PlayerState::STOP:
@@ -160,7 +141,7 @@ PlayerControl::LockSetPause(bool pause_flag) noexcept
 void
 PlayerControl::LockSetBorderPause(bool _border_pause) noexcept
 {
-	const std::lock_guard<Mutex> protect(mutex);
+	const std::scoped_lock protect{mutex};
 	border_pause = _border_pause;
 }
 
@@ -169,7 +150,7 @@ PlayerControl::LockGetStatus() noexcept
 {
 	PlayerStatus status;
 
-	std::unique_lock<Mutex> lock(mutex);
+	std::unique_lock lock{mutex};
 	if (!occupied && thread.IsDefined())
 		SynchronousCommand(lock, PlayerCommand::REFRESH);
 
@@ -193,19 +174,22 @@ PlayerControl::SetError(PlayerError type, std::exception_ptr &&_error) noexcept
 
 	error_type = type;
 	error = std::move(_error);
+
+	// TODO: is it ok to call this while holding mutex lock?
+	listener.OnPlayerError();
 }
 
 void
 PlayerControl::LockClearError() noexcept
 {
-	const std::lock_guard<Mutex> protect(mutex);
+	const std::scoped_lock protect{mutex};
 	ClearError();
 }
 
 void
 PlayerControl::LockSetTaggedSong(const DetachedSong &song) noexcept
 {
-	const std::lock_guard<Mutex> protect(mutex);
+	const std::scoped_lock protect{mutex};
 	tagged_song.reset();
 	tagged_song = std::make_unique<DetachedSong>(song);
 }
@@ -225,7 +209,7 @@ PlayerControl::ReadTaggedSong() noexcept
 std::unique_ptr<DetachedSong>
 PlayerControl::LockReadTaggedSong() noexcept
 {
-	const std::lock_guard<Mutex> protect(mutex);
+	const std::scoped_lock protect{mutex};
 	return ReadTaggedSong();
 }
 
@@ -235,7 +219,7 @@ PlayerControl::LockEnqueueSong(std::unique_ptr<DetachedSong> song) noexcept
 	assert(thread.IsDefined());
 	assert(song != nullptr);
 
-	std::unique_lock<Mutex> lock(mutex);
+	std::unique_lock lock{mutex};
 	EnqueueSongLocked(lock, std::move(song));
 }
 
@@ -294,7 +278,7 @@ PlayerControl::LockSeek(std::unique_ptr<DetachedSong> song, SongTime t)
 
 	assert(song != nullptr);
 
-	std::unique_lock<Mutex> lock(mutex);
+	std::unique_lock lock{mutex};
 	SeekLocked(lock, std::move(song), t);
 }
 
@@ -303,7 +287,7 @@ PlayerControl::SetCrossFade(FloatDuration duration) noexcept
 {
 	cross_fade.duration = std::max(duration, FloatDuration::zero());
 
-	idle_add(IDLE_OPTIONS);
+	listener.OnPlayerOptionsChanged();
 }
 
 void
@@ -311,7 +295,7 @@ PlayerControl::SetMixRampDb(float _mixramp_db) noexcept
 {
 	cross_fade.mixramp_db = _mixramp_db;
 
-	idle_add(IDLE_OPTIONS);
+	listener.OnPlayerOptionsChanged();
 }
 
 void
@@ -319,5 +303,5 @@ PlayerControl::SetMixRampDelay(FloatDuration _mixramp_delay) noexcept
 {
 	cross_fade.mixramp_delay = _mixramp_delay;
 
-	idle_add(IDLE_OPTIONS);
+	listener.OnPlayerOptionsChanged();
 }
