@@ -1,37 +1,19 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "ArgParser.hxx"
 #include "RangeArg.hxx"
 #include "Ack.hxx"
 #include "Chrono.hxx"
-#include "util/NumberParser.hxx"
+#include "lib/fmt/ToBuffer.hxx"
+#include "util/CNumberParser.hxx"
 
-#include <stdio.h>
 #include <stdlib.h>
 
 static inline ProtocolError
 MakeArgError(const char *msg, const char *value) noexcept
 {
-	char buffer[256];
-	snprintf(buffer, sizeof(buffer), "%s: %s", msg, value);
-	return ProtocolError(ACK_ERROR_ARG, buffer);
+	return {ACK_ERROR_ARG, FmtBuffer<256>("{}: {}", msg, value)};
 }
 
 uint32_t
