@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "SongSticker.hxx"
 #include "Sticker.hxx"
@@ -46,6 +30,24 @@ sticker_song_set_value(StickerDatabase &db,
 	db.StoreValue("song", uri.c_str(), name, value);
 }
 
+void
+sticker_song_inc_value(StickerDatabase &db,
+		       const LightSong &song,
+		       const char *name, const char *value)
+{
+	const auto uri = song.GetURI();
+	db.IncValue("song", uri.c_str(), name, value);
+}
+
+void
+sticker_song_dec_value(StickerDatabase &db,
+		       const LightSong &song,
+		       const char *name, const char *value)
+{
+	const auto uri = song.GetURI();
+	db.DecValue("song", uri.c_str(), name, value);
+}
+
 bool
 sticker_song_delete(StickerDatabase &db, const char *uri)
 {
@@ -108,6 +110,7 @@ void
 sticker_song_find(StickerDatabase &sticker_database, const Database &db,
 		  const char *base_uri, const char *name,
 		  StickerOperator op, const char *value,
+		  const char *sort, bool descending, RangeArg window,
 		  void (*func)(const LightSong &song, const char *value,
 			       void *user_data),
 		  void *user_data)
@@ -130,5 +133,6 @@ sticker_song_find(StickerDatabase &sticker_database, const Database &db,
 	data.base_uri_length = strlen(data.base_uri);
 
 	sticker_database.Find("song", data.base_uri, name, op, value,
+				  sort, descending, window,
 			      sticker_song_find_cb, &data);
 }
