@@ -1,26 +1,12 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "LocalStorage.hxx"
 #include "storage/StoragePlugin.hxx"
 #include "storage/StorageInterface.hxx"
 #include "storage/FileInfo.hxx"
+#include "input/InputStream.hxx"
+#include "input/LocalOpen.hxx"
 #include "fs/FileInfo.hxx"
 #include "fs/AllocatedPath.hxx"
 #include "fs/DirectoryReader.hxx"
@@ -66,6 +52,8 @@ public:
 
 	[[nodiscard]] std::string_view MapToRelativeUTF8(std::string_view uri_utf8) const noexcept override;
 
+	InputStreamPtr OpenFile(std::string_view uri_utf8, Mutex &mutex) override;
+
 private:
 	[[nodiscard]] AllocatedPath MapFSOrThrow(std::string_view uri_utf8) const;
 };
@@ -128,6 +116,16 @@ LocalStorage::MapToRelativeUTF8(std::string_view uri_utf8) const noexcept
 	return PathTraitsUTF8::Relative(base_utf8, uri_utf8);
 }
 
+InputStreamPtr
+LocalStorage::OpenFile(std::string_view uri_utf8, Mutex &mutex)
+{
+	auto path = MapFS(uri_utf8);
+	if (path == nullptr)
+		return nullptr;
+
+	return OpenLocalInputStream(path, mutex);
+}
+
 StorageFileInfo
 LocalStorage::GetInfo(std::string_view uri_utf8, bool follow)
 {
@@ -170,7 +168,8 @@ CreateLocalStorage(Path base_fs)
 	return std::make_unique<LocalStorage>(base_fs);
 }
 
-const StoragePlugin local_storage_plugin = {
+constexpr StoragePlugin local_storage_plugin = {
 	"local",
 	nullptr,
+	nullptr,
 };
