@@ -1,23 +1,7 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
-#include "Type.h"
+#include "Names.hxx"
 
 #include <cstdlib>
 #include <map>
@@ -47,6 +31,7 @@ main(int argc, char **argv)
 
 	fprintf(out,
 		"#include \"ParseName.hxx\"\n"
+		"#include \"Type.hxx\"\n"
 		"\n"
 		"#include <assert.h>\n"
 		"#include <string.h>\n"
@@ -60,10 +45,7 @@ main(int argc, char **argv)
 
 	char first = 0;
 
-	for (const auto &i : names) {
-		const std::string_view name = i.first;
-		const TagType tag = i.second;
-
+	for (const auto &[name, tag] : names) {
 		if (name.front() != first) {
 			if (first != 0)
 				fprintf(out, "    break;\n\n");
