@@ -1,25 +1,10 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "NullMixerListener.hxx"
-#include "mixer/MixerControl.hxx"
-#include "mixer/MixerList.hxx"
+#include "mixer/Control.hxx"
+#include "mixer/Mixer.hxx"
+#include "mixer/plugins/AlsaMixerPlugin.hxx"
 #include "filter/Registry.hxx"
 #include "event/Loop.hxx"
 #include "config/Block.hxx"
@@ -57,10 +42,7 @@ try {
 				 mixer_listener,
 				 ConfigBlock());
 
-	mixer_open(mixer);
-
-	volume = mixer_get_volume(mixer);
-	mixer_close(mixer);
+	volume = mixer->LockGetVolume();
 	mixer_free(mixer);
 
 	assert(volume >= -1 && volume <= 100);
