@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 /*
  * This program is a command line interface to MPD's PCM conversion
@@ -30,10 +14,9 @@
 #include "fs/Path.hxx"
 #include "fs/NarrowPath.hxx"
 #include "io/FileDescriptor.hxx"
-#include "util/ConstBuffer.hxx"
 #include "util/StaticFifoBuffer.hxx"
-#include "util/OptionDef.hxx"
-#include "util/OptionParser.hxx"
+#include "cmdline/OptionDef.hxx"
+#include "cmdline/OptionParser.hxx"
 #include "util/PrintException.hxx"
 #include "Log.hxx"
 #include "LogBackend.hxx"
@@ -83,7 +66,7 @@ ParseCommandLine(int argc, char **argv)
 	}
 
 	auto args = option_parser.GetRemaining();
-	if (args.size != 2)
+	if (args.size() != 2)
 		throw std::runtime_error("Usage: run_convert IN_FORMAT OUT_FORMAT <IN >OUT");
 
 	c.in_audio_format = ParseAudioFormat(args[0], false);
@@ -116,7 +99,7 @@ RunConvert(PcmConvert &convert, size_t in_frame_size,
 			const auto dest = buffer.Write();
 			assert(!dest.empty());
 
-			ssize_t nbytes = in_fd.Read(dest.data, dest.size);
+			ssize_t nbytes = in_fd.Read(dest);
 			if (nbytes <= 0)
 				break;
 
@@ -126,22 +109,22 @@ RunConvert(PcmConvert &convert, size_t in_frame_size,
 		auto src = buffer.Read();
 		assert(!src.empty());
 
-		src.size -= src.size % in_frame_size;
+		src = src.first(src.size() - src.size() % in_frame_size);
 		if (src.empty())
 			continue;
 
-		buffer.Consume(src.size);
+		buffer.Consume(src.size());
 
-		auto output = convert.Convert({src.data, src.size});
-		out_fd.FullWrite(output.data, output.size);
+		auto output = convert.Convert(src);
+		out_fd.FullWrite(output);
 	}
 
 	while (true) {
 		auto output = convert.Flush();
-		if (output.IsNull())
+		if (output.data() == nullptr)
 			break;
 
-		out_fd.FullWrite(output.data, output.size);
+		out_fd.FullWrite(output);
 	}
 }
 
