@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "encoder/EncoderList.hxx"
 #include "encoder/EncoderPlugin.hxx"
@@ -33,8 +17,6 @@
 
 #include <stddef.h>
 
-static uint8_t zero[256];
-
 int
 main([[maybe_unused]] int argc, [[maybe_unused]] char **argv)
 try {
@@ -61,7 +43,8 @@ try {
 
 	/* write a block of data */
 
-	encoder->Write(zero, sizeof(zero));
+	static constexpr std::byte zero[256]{};
+	encoder->Write(std::span{zero});
 
 	EncoderToOutputStream(os, *encoder);
 
@@ -86,7 +69,7 @@ try {
 
 	/* write another block of data */
 
-	encoder->Write(zero, sizeof(zero));
+	encoder->Write(std::span{zero});
 
 	/* finish */
 
